/****************************************************************/
/* AMR VAF MEASUREMENT                                          */
/* 1999 copyright Nortel Networks                               */
/* All Rights reserved                                          */
/* This C code may only be used for AMR-NS selection phase for  */
/* the assesment of the impact on VAF                           */
/* Filename : process.c                                         */
/****************************************************************/
/* File History                                                 */
/* DATE         AUTHOR          DESCRIPTION                     */
/* 09/99        F.GABIN         Creation                        */
/* 10/99        F.GABIN         exclude first 100 frames (2 sec)*/
/****************************************************************/
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "main.h"


/*-----------------------------------------------------------------------*
 *                          constants
 *-----------------------------------------------------------------------*/
#define FRAME_SIZE  250
enum TXFrameType { TX_SPEECH = 0,
                   TX_SID_FIRST,
                   TX_SID_UPDATE,
                   TX_NO_DATA,
                   TX_N_FRAMETYPES     /* number of frame types */
};


/*---------------------------------------------------------------------------
 *                                Types
 *--------------------------------------------------------------------------*/
typedef struct {
   int nbFrame;
   int nbSpeechFrame;
   int nbDtxSegment;
   int nbSidFrames;
} Stat;


/*---------------------------------------------------------------------------
 *                                Functions
 *--------------------------------------------------------------------------*/
static Stat *BitstreamStat(char *file);


/*---------------------------------------------------------------------------
 *                                Process
 *--------------------------------------------------------------------------*/
Report *Process(char *file, char *Mode)
{
   static Report report;
   FILE *f;
   char commandLine[500];
   Stat *stat;

/*
 * check that file exists
 */
   f = fopen(file, "rb");
   if(!f) {
      fprintf(stderr, "%s file doesn't exists\n", file);
      exit(1);
   }
   fclose(f);

#if (TEST == 1)   /* For file search test */

   /* return dummy report */
report.nbFrame        = 100;
report.nbSpeechFrame1 = 70;
report.nbDtxSegment1  = 4;
report.nbSidFrames1   = 10;
report.nbSpeechFrame2 = 70;
report.nbDtxSegment2  = 4;
report.nbSidFrames2   = 10;

return(&report);

#else //
/*
 * run encoder vad1
 */
   strcpy(commandLine, COMMAND_LINE_VAD1);
   strcat(commandLine, " ");
   strcat(commandLine, Mode);
   strcat(commandLine, " ");
   strcat(commandLine, file);
   strcat(commandLine, " bit.tmp");
   system(commandLine);

/*
 * statistics on coded bitstream
 * fill report
 */
   stat = BitstreamStat("bit.tmp");
   report.nbFrame        = stat->nbFrame;
   report.nbSpeechFrame1 = stat->nbSpeechFrame;
   report.nbDtxSegment1  = stat->nbDtxSegment;
   report.nbSidFrames1   = stat->nbSidFrames;

/*
 * run encoder vad2
 */
   strcpy(commandLine, COMMAND_LINE_VAD2);
   strcat(commandLine, " ");
   strcat(commandLine, Mode);
   strcat(commandLine, " ");
   strcat(commandLine, file);
   strcat(commandLine, " bit.tmp");
   system(commandLine);

/*
 * statistics on coded bitstream
 * fill report
 */
   stat = BitstreamStat("bit.tmp");
   assert(report.nbFrame == stat->nbFrame);
   report.nbSpeechFrame2 = stat->nbSpeechFrame;
   report.nbDtxSegment2  = stat->nbDtxSegment;
   report.nbSidFrames2   = stat->nbSidFrames;

   return(&report);
#endif
}



/* ======================================================================== */



/*---------------------------------------------------------------------------
 *                            BitstreamStat
 *--------------------------------------------------------------------------*/
Stat *BitstreamStat(char *file)
{
   static Stat stat;
   FILE *bitstream;
   int nbFrame = 0;
   int nbSidFrame = 0;
   int nbSpeechFrame = 0;
   int nbDtxSegment = 0;
   short buffer[FRAME_SIZE];


/*
 * open bitstream
 */
   bitstream = fopen(file, "rb");
	if(!bitstream) {
		fprintf(stderr, "unable to open bitstream %s\n", file);
      exit(1);
   }


/*
 * while EOF not reached
 *    read a frame from bitstream
 *    increment totFrame
 *    check frame type
 *    increment sidFrame
 */
   for(;;) {
      short frameType;
      long l = fread(buffer, 2, FRAME_SIZE, bitstream);
      if(l == 0)
         break;
      if(l != FRAME_SIZE) {
         fprintf(stderr, "incomplete bitstream (%s)\n", file);
         exit(1);
      }


      nbFrame++;

      if(nbFrame >= 100) {
          switch(frameType = buffer[0]) {
          case TX_SPEECH:
             nbSpeechFrame++;
             break;

          case TX_SID_FIRST:
             nbDtxSegment++;
             break;

          case TX_SID_UPDATE:
             nbSidFrame++;
             break;

          case TX_NO_DATA:
             break;

          default:
             fprintf(stderr, "unknown frame type (%d) in bitstream %s (frame %d)\n", frameType, file, nbFrame);
             exit(1);
          }
      }
   }


/*
 * close file
 * fill stat
 */
   fclose(bitstream);
   if(nbFrame <= 100) {
      fprintf(stderr, "bitstream is too short (less than 2 seconds)(%s)\n", file);
      exit(1);
   }

   stat.nbFrame       = nbFrame - 100;
   stat.nbSpeechFrame = nbSpeechFrame;
   stat.nbDtxSegment  = nbDtxSegment;
   stat.nbSidFrames   = nbSidFrame;
   return(&stat);
}
